"""init

Revision ID: 7dea8c4ee39f
Revises: 
Create Date: 2025-09-09 14:30:52.222907

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '7dea8c4ee39f'
down_revision: Union[str, Sequence[str], None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('bind_domains',
    sa.Column('domain', sa.String(), nullable=False),
    sa.Column('stack_fqdn', sa.String(), nullable=False),
    sa.PrimaryKeyConstraint('domain', 'stack_fqdn')
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('bind_domains')
    # ### end Alembic commands ###
